@@ -28,7 +28,6 @@ __requires__ = ['SQLAlchemy >= 0.7']
 import pkg_resources
 
 import datetime
-import functools
 import hashlib
 import json
 import logging
@@ -45,28 +44,29 @@ from sqlalchemy.orm import scoped_session
 from sqlalchemy.orm import relation
 from sqlalchemy.orm import backref
 
-import fedmsg.utils
+import fedmsg
 
 import fmn.lib.defaults
 
-BASE = declarative_base()
+
+class FMNBase(object):
+    def notify(self, openid, context, changed):
+        obj = type(self).__name__.lower()
+        topic = obj + ".update"
+        fedmsg.publish(
+            topic=topic,
+            msg=dict(
+                openid=openid,
+                context=context,
+                changed=changed,
+            )
+        )
+
+
+BASE = declarative_base(cls=FMNBase)
 
 log = logging.getLogger(__name__)
 
-execute_error_template = """
-exception
----------
-%s
-
-rule
-----
-%r
-
-message
--------
-%s
-"""
-
 
 def init(db_url, alembic_ini=None, debug=False, create=False):
     """ Create the tables in the database using the information from the
@@ -115,6 +115,16 @@ class Context(BASE):
     icon = sa.Column(sa.String(32), nullable=False)
     placeholder = sa.Column(sa.String(256))
 
+    def __json__(self):
+        return {
+            'name': self.name,
+            'detail_name': self.detail_name,
+            'description': self.description,
+            'created_on': self.created_on,
+            'icon': self.icon,
+            'placeholder': self.placeholder,
+        }
+
     def get_confirmation(self, openid):
         for confirmation in self.confirmations:
             if confirmation.openid == openid:
@@ -155,29 +165,6 @@ class Context(BASE):
         session.commit()
         return context
 
-    def _recipients(self, session, config, valid_paths, message):
-        """ Returns the list of recipients for a message. """
-
-        for user in User.all(session):
-            pref = Preference.load(session, user, self)
-            if not pref or not pref.detail_values:
-                continue
-
-            flter = pref.prefers(session, config, valid_paths, message)
-            if not flter:
-                continue
-
-            for value in pref.detail_values:
-                yield {
-                    'user': user.openid,
-                    pref.context.detail_name: value.value,
-                    'filter_name': flter.name,
-                    'filter_id': flter.id,
-                }
-
-    def recipients(self, session, config, valid_paths, message):
-        return list(self._recipients(session, config, valid_paths, message))
-
 
 class User(BASE):
     __tablename__ = 'users'
@@ -188,6 +175,13 @@ class User(BASE):
     api_key = sa.Column(sa.Text)
     created_on = sa.Column(sa.DateTime, default=datetime.datetime.utcnow)
 
+    def __json__(self):
+        return {
+            'openid': self.openid,
+            'openid_url': self.openid_url,
+            'created_on': self.created_on,
+        }
+
     def __repr__(self):
         return "<fmn.lib.models.User %r %r>" % (self.openid, self.openid_url)
 
@@ -246,6 +240,13 @@ class Rule(BASE):
     # JSON-encoded kw
     _arguments = sa.Column(sa.String(256))
 
+    def __json__(self):
+        return {
+            'created_on': self.created_on,
+            'code_path': self.code_path,
+            'arguments': self.arguments,
+        }
+
     def __repr__(self):
         return "<fmn.lib.models.Rule %r(**%r)>" % (
             self.code_path, self.arguments)
@@ -259,13 +260,6 @@ class Rule(BASE):
         if not kw is None:
             self._arguments = json.dumps(kw)
 
-    def _instantiate_callable(self):
-        # This is a bit of a misnomer, load_class can load anything.
-        fn = fedmsg.utils.load_class(str(self.code_path))
-        # Now, partially apply our keyword arguments.
-        fn = functools.partial(fn, **self.arguments)
-        return fn
-
     @staticmethod
     def validate_code_path(valid_paths, code_path, **kw):
         """ Raise an exception if code_path is not one of our
@@ -301,29 +295,6 @@ class Rule(BASE):
         else:
             return valid_paths[root][name]['doc']
 
-    def execute(self, session, config, valid_paths, message):
-        """ Load our callable and execute it.
-
-        Note, we validate the code_path again here for the second time.  Once
-        before it is inserted into the db, and once again before we execute it.
-        This is mitigation in case other code is vulnerable to injecting
-        arbitrary data into the db.
-        """
-
-        Rule.validate_code_path(
-            valid_paths, self.code_path, **self.arguments)
-
-        fn = self._instantiate_callable()
-        try:
-            return fn(config, message)
-        except Exception as e:
-            log.error(execute_error_template % (
-                traceback.format_exc(),
-                self,
-                pprint.pformat(message),
-            ))
-            return False
-
 
 class Filter(BASE):
     __tablename__ = 'filters'
@@ -336,6 +307,14 @@ class Filter(BASE):
         sa.ForeignKey('preferences.id'))
     preference = relation('Preference', backref=backref('filters'))
 
+    def __json__(self):
+        return {
+            'id': self.id,
+            'name': self.name,
+            'created_on': self.created_on,
+            'rules': [r.__json__() for r in self.rules]
+        }
+
     def __repr__(self):
         return "<fmn.lib.models.Filter %r>" % (self.name)
 
@@ -356,6 +335,11 @@ class Filter(BASE):
         self.rules.append(rule)
         session.flush()
         session.commit()
+
+        pref = self.preference
+        if pref:
+            self.notify(pref.openid, pref.context_name, "rules")
+
         return rule
 
     def remove_rule(self, session, code_path, **kw):
@@ -363,28 +347,15 @@ class Filter(BASE):
             if r.code_path == code_path:
                 session.delete(r)
                 session.commit()
+
+                pref = self.preference
+                if pref:
+                    self.notify(pref.openid, pref.context_name, "rules")
+
                 return
 
         raise ValueError("No such rule found: %r" % code_path)
 
-    def matches(self, session, config, paths, message):
-        """ Return true if this filter matches the given message.
-
-        This is the case if *all* of the associated rules match.
-
-        ...with one exception.  If no rules are defined, the filter does not
-        match (even though technically, all of its zero rules match).
-        """
-
-        if not self.rules:
-            return False
-
-        for rule in self.rules:
-            if not rule.execute(session, config, paths, message):
-                return False
-
-        return True
-
 
 class DetailValue(BASE):
     __tablename__ = 'detail_values'
@@ -435,6 +406,11 @@ class Preference(BASE):
     # forcing new users into an opt-out situation.
     enabled = sa.Column(sa.Boolean, default=False, nullable=False)
 
+    # Various presentation booleans
+    markup_messages = sa.Column(sa.Boolean, default=False)
+    triggered_by_links = sa.Column(sa.Boolean, default=False)
+    shorten_links = sa.Column(sa.Boolean, default=False)
+
     openid = sa.Column(
         sa.Text,
         sa.ForeignKey('users.openid'),
@@ -451,6 +427,21 @@ class Preference(BASE):
         sa.UniqueConstraint('openid', 'context_name'),
     )
 
+    def __json__(self):
+        return {
+            'created_on': self.created_on,
+            'batch_delta': self.batch_delta,
+            'batch_count': self.batch_count,
+            'markup_messages': self.markup_messages,
+            'triggered_by_links': self.triggered_by_links,
+            'shorten_links': self.shorten_links,
+            'enabled': self.enabled,
+            'context': self.context.__json__(),
+            'user': self.user.__json__(),
+            'filters': [f.__json__() for f in self.filters],
+            'detail_values': [v.value for v in self.detail_values],
+        }
+
     @property
     def should_batch(self):
         """ If the user has any batching preferences at all, then we should """
@@ -469,6 +460,25 @@ class Preference(BASE):
         self.batch_count = count
         session.add(self)
         session.commit()
+        self.notify(self.openid, self.context_name, "batch_values")
+
+    def set_markup_messages(self, session, value):
+        self.markup_messages = value
+        session.add(self)
+        session.commit()
+        self.notify(self.openid, self.context_name, "markup_messages")
+
+    def set_triggered_by_links(self, session, value):
+        self.triggered_by_links = value
+        session.add(self)
+        session.commit()
+        self.notify(self.openid, self.context_name, "triggered_by_links")
+
+    def set_shorten_links(self, session, value):
+        self.shorten_links = value
+        session.add(self)
+        session.commit()
+        self.notify(self.openid, self.context_name, "shorten_links")
 
     @classmethod
     def by_user(cls, session, openid):
@@ -548,6 +558,7 @@ class Preference(BASE):
         session.delete(value)
         session.flush()
         session.commit()
+        self.notify(self.openid, self.context_name, "details")
 
     def update_details(self, session, detail_value):
         log.debug("Adding %r to %r" % (detail_value, self))
@@ -556,16 +567,25 @@ class Preference(BASE):
         self.detail_values.append(value)
         session.flush()
         session.commit()
+        self.notify(self.openid, self.context_name, "details")
 
     def set_enabled(self, session, enabled):
         self.enabled = enabled
         session.flush()
         session.commit()
+        self.notify(self.openid, self.context_name, "enabled")
+
+    def delete_filter(self, session, filter_name):
+        filter = self.get_filter_name(session, filter_name)
+        session.delete(filter)
+        session.commit()
+        self.notify(self.openid, self.context_name, "filters")
 
     def add_filter(self, session, filter):
         self.filters.append(filter)
         session.flush()
         session.commit()
+        self.notify(self.openid, self.context_name, "filters")
 
     def has_filter_name(self, session, filter_name):
         for filter in self.filters:
@@ -595,22 +615,6 @@ class Preference(BASE):
 
         raise ValueError("No such filter %r" % filter_id)
 
-    def prefers(self, session, config, valid_paths, message):
-        """ Evaluate to true if this preference "prefers" this message.
-
-        That is the case if *any* of the associated filters match.
-
-        The first filter that matches the message is returned for bookkeeping.
-
-        If no filter matches, None is returned.
-        """
-
-        for filter in self.filters:
-            if filter.matches(session, config, valid_paths, message):
-                return filter
-
-        return None
-
 
 def hash_producer(*args, **kwargs):
     """ Returns a random hash for a confirmation secret. """
@@ -725,6 +729,7 @@ class Confirmation(BASE):
         self.detail_value = value
         session.flush()
         session.commit()
+        self.notify(self.openid, self.context_name, "value")
 
     def set_status(self, session, status):
         assert status in self.STATUSES
@@ -738,6 +743,7 @@ class Confirmation(BASE):
 
         session.flush()
         session.commit()
+        self.notify(self.openid, self.context_name, "status")
 
 
 class QueuedMessage(BASE):
